/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.bag.action;

import com.iohao.game.action.skeleton.annotation.ActionController;
import com.iohao.game.action.skeleton.annotation.ActionMethod;
import com.iohao.game.action.skeleton.core.flow.FlowContext;
import com.iohao.mmo.bag.cmd.BagCmd;
import com.iohao.mmo.bag.entity.Bag;
import com.iohao.mmo.bag.entity.BagItem;
import com.iohao.mmo.bag.mapper.BagMapper;
import com.iohao.mmo.bag.proto.BagItemMessage;
import com.iohao.mmo.bag.proto.BagMessage;
import com.iohao.mmo.bag.service.BagService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Component;

/**
 * 背包
 *
 * @author 渔民小镇
 * @date 2023-08-04
 */
@Component
@ActionController(BagCmd.cmd)
public class BagAction {
    @Resource
    BagService bagService;

    /**
     * 查询玩家背包
     *
     * @return 背包
     */
    @ActionMethod(BagCmd.bag)
    public BagMessage getBag(FlowContext flowContext) {
        long userId = flowContext.getUserId();
        Bag bag = bagService.ofBag(userId);

        return BagMapper.ME.convert(bag);
    }

    /**
     * 往背包添加（增加）物品
     *
     * @param bagItemMessage 物品
     * @param flowContext    flowContext
     * @return 当前物品的最新信息
     */
    @ActionMethod(BagCmd.incrementItem)
    public BagItemMessage internalIncrementItem(BagItemMessage bagItemMessage, FlowContext flowContext) {
        long userId = flowContext.getUserId();

        BagItem bagItem = BagMapper.ME.convert(bagItemMessage);

        bagItem = bagService.incrementItem(bagItem, userId);
        BagMapper.ME.to(bagItem, bagItemMessage);

        return bagItemMessage;
    }

    /**
     * 从背包减少物品
     *
     * @param bagItemMessage 物品
     * @param flowContext    flowContext
     * @return 当前物品的最新信息
     */
    @ActionMethod(BagCmd.decrementItem)
    public BagItemMessage internalDecrementItem(BagItemMessage bagItemMessage, FlowContext flowContext) {
        long userId = flowContext.getUserId();

        BagItem bagItem = BagMapper.ME.convert(bagItemMessage);

        bagItem = bagService.decrementItem(bagItem, userId);
        BagMapper.ME.to(bagItem, bagItemMessage);

        return bagItemMessage;
    }
}
