package com.iohao.mmo.equip.utils;

import org.springframework.stereotype.Component;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.ThreadLocalRandom;

/**
 * @author kioor
 * @ClassName RandomUtils
 * @description: 随机概率工具
 *  常见四种概率算法：
 *      1.常规做法，直接配置概率，程序直接判定
 *      2.在1的基础上，加个保底次数，当连续不发生的次数高于保底时，强制发生
 *      3.设置基础概率，事件不发生概率翻倍
 *      4.设置数组，将事件发生概率变成数组元素
 * @date 2023年07月26日
 * @version: 1.0
 */
@Component
public class RandomUtils {

    private static final ThreadLocalRandom RANDOM = ThreadLocalRandom.current();

    ////////////////////////// ↓↓↓算法实现↓↓↓ ///////////////////////////
    /**
     * 由概率随机是否触发（给定一个概率，返回是否命中）
     *
     * @param chance 概率 0.000000 - 1.000000
     * @return boolean
     */
    public static boolean isLuck(BigDecimal chance) {
        int chanceInt = chance.multiply(new BigDecimal("1000000")).intValue();
        int randomNum = RANDOM.nextInt(1000000);
        return randomNum <= chanceInt;
    }

    /**
     * 由概率随机一个值（给定一个范围，范围随机值）
     *
     * @param rangeMin 限定范围的下限
     * @param rangeMax 限定范围的上限
     * @return boolean
     */
    public static BigDecimal randomFromArr(BigDecimal rangeMin,BigDecimal rangeMax) {
        Random random = new Random();
        BigDecimal randomValue =
                rangeMin.add(
                        rangeMax.subtract(rangeMin)
                                .multiply(new BigDecimal(random.nextDouble()))
                );
        return randomValue;
    }


    /**
     * 由概率随机一个值（给定一个范围，范围随机值）
     *
     * @param randomMin 限定范围的最小值(int)
     * @param randomMax 限定范围的最大值(int)
     * @return boolean
     */
    public static int randomFromInt(int randomMin,int randomMax) {
        BigDecimal rangeMin = new BigDecimal(randomMin);
        BigDecimal rangeMax = new BigDecimal(randomMax);
        return RandomUtils.randomFromArr(rangeMin,rangeMax).intValue();

    }



//    ///////////////////////// ↓↓↓业务相关↓↓↓ ////////////////////
//    /**
//     * 怪物掉落物品计算（对象列表）-无排斥随机，每一次对每一个对象随机概率都是公平的
//     * @param monster2GoodsDTOList 对象列表
//     * @return 筛选后的对象列表
//     */
//    public List<Monster2GoodsDTO> obtainGoods(List<Monster2GoodsDTO> monster2GoodsDTOList){
//        List<Monster2GoodsDTO> backMonster2GoodsDTOList = new ArrayList<>();
//        for(Monster2GoodsDTO monster2GoodsDTO:monster2GoodsDTOList){
//            if(isLuck(monster2GoodsDTO.getGoodsProb())){
//                backMonster2GoodsDTOList.add(monster2GoodsDTO);
//            }
//        }
//        return backMonster2GoodsDTOList;
//    }

}
